/*
 * File access convenience functions and types.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>

#include <systypes/base.h>



EXTERN_C_BEGIN

/* File access abstraction. */

typedef struct
{
  /* File object reference. */

  l4_cap_idx_t ref;

  /* Notification IRQ reference. */

  l4_cap_idx_t irq;

  /* Mapped memory accessing a file region. */

  char *memory;

  /* File region parameters. */

  offset_t start_pos, end_pos;  /* start and end positions of region */
  offset_t data_end;            /* amount/extent of data in the region */
  offset_t data_current;        /* client access offset */

  /* Total size of file. */

  offset_t size;

  /* Arbitrary memory mapping support. */

  int can_mmap;

  /* Explicit size support. */

  int has_size;

  /* Blocking accesses. */

  int can_block;

} file_t;



/* File operations. */

void file_close(file_t *file);
long file_open(file_t *file, const char *filename, flags_t flags, l4_cap_idx_t server);

/* File lifecycle operations. */

long file_context(file_t *file, l4_cap_idx_t server);
long file_context_open(file_t *file, flags_t flags, file_t *context);
void file_init(file_t *file);

/* File and region operations. */

long file_flush(file_t *file);
long file_mmap(file_t *file, offset_t position, offset_t length);
long file_resize(file_t *file, offset_t size);

/* File and region properties. */

offset_t file_populated_span(file_t *file);
offset_t file_span(file_t *file);

/* Convenience functions. */

char *file_string_get(file_t *file, offset_t offset);
int file_string_set(file_t *file, const char *data, offset_t offset, offset_t *written);

/* Client data functions. */

offset_t file_data_available(file_t *file);
char *file_data_current(file_t *file);
offset_t file_data_current_position(file_t *file);
offset_t file_data_end_position(file_t *file);
offset_t file_data_space(file_t *file);

/* Client data transfer functions. */

void file_data_read(file_t *file, char *buf, offset_t to_transfer);
void file_data_write(file_t *file, char *buf, offset_t to_transfer);

/* Notification functions. */

long file_notify_subscribe(file_t *file, flags_t flags);
long file_notify_unsubscribe(file_t *file);
long file_notify_wait(file_t *file);



/* Pipe operations. */

long pipe_open(offset_t size, file_t *reader, file_t *writer, l4_cap_idx_t server);

/* Pipe region operations. */

long pipe_current(file_t *pipe);
long pipe_next(file_t *pipe);
long pipe_written(file_t *pipe, offset_t size);



EXTERN_C_END

// vim: tabstop=2 expandtab shiftwidth=2
